/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.bonita.facade.def.element.impl;

/**
 * Utility class to create a Deployment object containing the the XPDL file and optionally its
 */
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.facade.def.element.BusinessArchive;
import org.ow2.bonita.facade.def.element.Resource;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.Misc;

/**
 * This class implements a deployment object. This object can be provided to one of the method of the ManagementAPI
 * {@link org.ow2.bonita.facade.ManagementAPI#deploy(BusinessArchiveImpl) method} to allow the deployment of a XPDL file
 * and optionnally its depending java classes for hooks, performer assignments, mappers...
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class BusinessArchiveImpl implements Serializable, BusinessArchive {

  /**
   * 
   */
  private static final long serialVersionUID = 218650935088396315L;

  private static final Logger LOG = Logger.getLogger(BusinessArchiveImpl.class.getName());

  protected Resource xpdlFile = null;
  protected Collection<Resource> classes = new HashSet<Resource>();
  protected Collection<Resource> jars = new HashSet<Resource>();
  protected Map<String, Resource> resources = new HashMap<String, Resource>();

  protected BusinessArchiveImpl() { }

  public BusinessArchiveImpl(Map<String, byte[]> resources) throws IOException {
    Misc.checkArgsNotNull(resources);
    for (Map.Entry<String, byte[]> resource : resources.entrySet()) {
      addResource(resource.getKey(), resource.getValue());
    }
  }

  public BusinessArchiveImpl(BusinessArchive businessArchive) {
    if (LOG.isLoggable(Level.FINEST)) {
      LOG.finest("Building a Deployment object from businessArchive: " + classes);
    }
    for (Resource resource : businessArchive.getResources()) {
      addResource(resource);
    }
  }

  public Resource getXpdlFile() {
    return this.xpdlFile;
  }
  public Collection<Resource> getClasses() {
    return classes;
  }
  public Collection<Resource> getJarFiles() {
    return jars;
  }
  public Resource getResource(String resourcePath) {
    return resources.get(resourcePath);
  }
  public Collection<Resource> getResources(String regex) {
    Set<String> keys = resources.keySet();
    Collection<Resource> result = new ArrayList<Resource>();
    for (String key : keys) {
      if (key.matches(regex)) {
        result.add(resources.get(key));
      }
    }
    return result;
  }
  public void addResource(String resourcePath, byte[] resourceData) {
    Misc.checkArgsNotNull(resourcePath, resourceData);
    Resource resource = new ResourceImpl(resourcePath, resourceData);
    this.resources.put(resourcePath, resource);

    if (resourcePath != null) {
      if (resourcePath.endsWith(".xpdl")) {
        if (this.xpdlFile != null) {
          throw new BonitaRuntimeException("The given bar archive contains many '.xpdl' files");
        }
        this.xpdlFile = resource;
      } else if (resourcePath.endsWith(".jar")) {
        this.jars.add(resource);
      } else if (resourcePath.endsWith(".class")) {
        this.classes.add(resource);
      }
    }
  }

  public void addResource(Resource resource) {
    addResource(resource.getPath(), resource.getData());
  }

  public Collection<Resource> getResources() {
    return resources.values();
  }

}
